package cn.com.qmth.stmms.biz.report.service.impl;

import java.util.LinkedList;
import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectQuestionDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectQuestion;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectQuestionService;

/**
 * 客、主观题分析 service 实现
 *
 * @author xuwenjin
 * @Date 2019-04-17
 */
@Service
public class ReportSubjectQuestionServiceImpl extends BaseQueryService<ReportSubjectQuestion> implements
        ReportSubjectQuestionService {

    @Autowired
    private ReportSubjectQuestionDao reportSubjectQuestionDao;

    @Override
    public ReportSubjectQuestion save(ReportSubjectQuestion reportSubjectQuestion) {
        return reportSubjectQuestionDao.save(reportSubjectQuestion);
    }

    @Transactional
    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectQuestionDao.deleteByExamId(examId);

    }

    @Override
    public List<ReportSubjectQuestion> findByQuery(final ReportSubjectQuery query) {
        query.setSort(new Sort(Direction.ASC, "paperType", "mainNumber", "subNumber"));
        Page<ReportSubjectQuestion> result = reportSubjectQuestionDao.findAll(
                new Specification<ReportSubjectQuestion>() {

                    @Override
                    public Predicate toPredicate(Root<ReportSubjectQuestion> root, CriteriaQuery<?> cQuery,
                            CriteriaBuilder cb) {
                        List<Predicate> predicates = new LinkedList<Predicate>();
                        if (query.getExamId() != null) {
                            predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                        }
                        if (StringUtils.isNotBlank(query.getSubjectCode())) {
                            predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                        }
                        if (query.getMainNumber() != null) {
                            predicates.add(cb.equal(root.get("mainNumber"), query.getMainNumber()));
                        }
                        if (query.getSubNumber() != null) {
                            predicates.add(cb.equal(root.get("subNumber"), query.getSubNumber()));
                        }
                        if (query.getObjective() != null) {
                            predicates.add(cb.equal(root.get("objective"), query.getObjective()));
                        }
                        if (query.getObjective() != null && query.getObjective()
                                && StringUtils.isNotBlank(query.getPaperType())) {
                            predicates.add(cb.equal(root.get("paperType"), query.getPaperType()));
                        }
                        if (query.getNullPaperType() != null && query.getNullPaperType().booleanValue()) {
                            predicates.add(cb.isNull(root.get("paperType")));
                        }
                        return predicates.isEmpty() ? cb.conjunction() : cb.and(predicates
                                .toArray(new Predicate[predicates.size()]));
                    }
                }, query);
        return result.getContent();
    }

    @Override
    public ReportSubjectQuestion findOne(Integer examId, String subjectCode, boolean objective, String paperType,
            Integer mainNumber, Integer subNumber) {
        ReportSubjectQuery query = new ReportSubjectQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setObjective(objective);
        query.setPaperType(paperType);
        query.setMainNumber(mainNumber);
        query.setSubNumber(subNumber);
        List<ReportSubjectQuestion> list = findByQuery(query);
        return list.isEmpty() ? null : list.get(0);
    }

    @Override
    public List<ReportSubjectQuestion> findByExamIdAndSubjectCodeAndObjectiveAndPaperType(Integer examId,
            String subjectCode, boolean objective, String paperType) {
        return reportSubjectQuestionDao.findByExamIdAndSubjectCodeAndObjectiveAndPaperType(examId, subjectCode,
                objective, paperType, new Sort(Direction.ASC, "mainNumber", "subNumber"));
    }

}
